/* Generic array accessing code */

#include <stdlib.h>
#include "gah.h"

struct gah *new_gah (void *b, int t, int dc)
{
  struct gah *result = (struct gah *) malloc(sizeof(struct gah) + (dc-1)*sizeof(long));

  result->base = b;
  result->type = t;
  result->dimcount = dc;

  return result;
}

long get_gah_dimension(const struct gah *g, int d)
{
  return (&(g->dim))[d];
}

void set_gah_dimension(struct gah *g, int d, long v)
{
  (&(g->dim))[d] = v;
}

long gah_offset(const struct gah *g, const long *vec)
{
  long result = 0;
  long dsize = 1;
  int i;

  for(i = g->dimcount-1; i>=0; i--) {
    result += vec[i] * dsize;
    dsize *= get_gah_dimension(g, i);
  }
  return result;
}

double get_gah_value(const struct gah *g, const long *vec)
{
  long offset = gah_offset(g, vec);
  double result;

  switch(g->type) {

  case HE5T_NATIVE_DOUBLE: result = ((double *)g->base)[offset];
    break;
  case HE5T_NATIVE_FLOAT: result = ((float *)g->base)[offset];
    break;
  case HE5T_NATIVE_INT: result = ((int *)g->base)[offset];
    break;
  case HE5T_NATIVE_UINT: result = ((unsigned int *)g->base)[offset];
    break;
  case HE5T_NATIVE_SHORT: result = ((short *)g->base)[offset];
    break;
  case HE5T_NATIVE_USHORT: result = ((unsigned short *)g->base)[offset];
    break;
  case HE5T_NATIVE_SCHAR: result = ((char *)g->base)[offset];
    break;
  case HE5T_NATIVE_UCHAR: result = ((unsigned char *)g->base)[offset];
    break;
  default:
    /* scream about an error here */
    break;
  }
  return result;
}


void set_gah_value(struct gah *g, const long *vec, double v)
{
  long offset = gah_offset(g, vec);

  switch(g->type) {

  case HE5T_NATIVE_DOUBLE: ((double *)g->base)[offset] = v;
    break;
  case HE5T_NATIVE_FLOAT: ((float *)g->base)[offset] = v;
    break;
  case HE5T_NATIVE_INT: ((int *)g->base)[offset] = v;
    break;
  case HE5T_NATIVE_UINT: ((unsigned int *)g->base)[offset] = v;
    break;
  case HE5T_NATIVE_SHORT: ((short *)g->base)[offset] = v;
    break;
  case HE5T_NATIVE_USHORT: ((unsigned short *)g->base)[offset] = v;
    break;
  case HE5T_NATIVE_SCHAR: ((char *)g->base)[offset] = v;
    break;
  case HE5T_NATIVE_UCHAR: ((unsigned char *)g->base)[offset] = v;
    break;
  default:
    /* scream about an error here */
    break;
  }
}

/* Makes the indices in vec point to the "next" element in the array handled
   by g.  Returns false if there is no "next" element.  Does not iterate
   values with a corresponding non-zero mask byte.
*/
int gah_iter(const struct gah *g, long *vec, const char *mask)
{
  long index = g->dimcount -1;
  int result = 0;

  while (!result && index >= 0) {
    if(mask == NULL || mask[index] == 0) {
      if(++vec[index] >= get_gah_dimension(g, index)) {
	vec[index] = 0;
      }
      else
	result = 1;
    }
      index--;
  }
  return result;
}
